@@ -3,7 +3,7 @@
 from ..utils.masked import masked
 from ..utils.common import grouper
 from .serializable import Serializable
-from .datatypes import DataTypes, DataTypeNames
+from .datatypes import DataTypes, DataTypeNames, Vector, Vector2D, Quaternion
 from .odict import ODict
 from .composite import Composite
 from .listener import DatasetEventSender, ParameterListener, DatasetListener, DatasetEvent, EventTypeOf
@@ -20,7 +20,7 @@ from ..utils.common import exprstrs, wls, mstr, t2l
 from tabulate import tabulate
 
 import builtins
-from collections import OrderedDict
+from collections import OrderedDict, Sequence
 from numbers import Number
 import logging
 # create logger
@@ -76,7 +76,12 @@ def parameterDataClasses(tt):
 
 class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Serializable):
     """ Parameter is the interface for all named attributes
-    in the MetaData container. It can have a value and a description.
+    in the MetaData container.
+
+    A Parameter is a variable with associated information about its description, unit, type, valid ranges, default, format code etc. Type can be numeric, string, datetime, vector.
+
+    Often a parameter shows a property. So a parameter in the metadata of a dataset or product is often called a property.
+
     Default     value=None, description='UNKNOWN'
     """
 
@@ -541,6 +546,38 @@ class NumericParameter(Parameter, Quantifiable):
                            typecode=self._typecode,
                            _STID=self._STID)
 
+    def setValue(self, value):
+        """ accept any type that a Vector does.
+        """
+        if value is not None and issubclass(value.__class__, Sequence):
+            d = list(value)
+            if len(d) == 2:
+                value = Vector2D(d)
+            elif len(d) == 3:
+                value = Vector(d)
+            elif len(d) == 4:
+                value = Quaternion(d)
+            else:
+                raise ValueError(
+                    'Sequence of only 2 to 4 elements for NumericParameter')
+        super().setValue(value)
+
+    def setDefault(self, default):
+        """ accept any type that a Vector does.
+        """
+        if default is not None and issubclass(default.__class__, Sequence):
+            d = list(default)
+            if len(d) == 2:
+                default = Vector2D(d)
+            elif len(d) == 3:
+                default = Vector(d)
+            elif len(d) == 4:
+                default = Quaternion(d)
+            else:
+                raise ValueError(
+                    'Sequence of only 2 to 4 elements for NumericParameter')
+        super().setDefault(default)
+
     __hash__ = DeepEqual.__hash__
 
 
