@@ -10,7 +10,8 @@ from ..utils.common import pathjoin, trbk
 import filelock
 import sys
 import shutil
-import pdb
+import mmap
+import time
 import json
 import os
 from os import path as op
@@ -39,19 +40,6 @@ class ODEncoder(json.JSONEncoder):
         return d
 
 
-def writeJsonwithbackup(fp, data):
-    """ write data in JSON after backing up the existing one.
-    """
-    if op.exists(fp):
-        os.rename(fp, fp + '.old')
-    # js = json.dumps(data, cls=ODEncoder)
-    #logger.debug('Writing %s stat %s' % (fp, str(os.path.exists(fp+'/..'))))
-    js = serialize(data)
-    with open(fp, mode="w+") as f:
-        f.write(js)
-    logger.debug('JSON saved to: ' + fp)
-
-
 def wipeLocal(poolpath):
     """
     does the scheme-specific remove-all
@@ -82,9 +70,21 @@ class LocalPool(ProductPool):
         """
         # print(__name__ + str(kwds))
         super(LocalPool, self).__init__(**kwds)
+
+    def setup(self):
+        """ Sets up LocalPool interals.
+
+        Make sure that self._poolname and self._poolurl are present.
+        """
+
+        if not hasattr(self, '_poolname') or self._poolname is None or \
+           not hasattr(self, '_poolurl') or self._poolurl is None:
+            return
+
         real_poolpath = self.transformpath(self._poolname)
         if not op.exists(real_poolpath):
             os.makedirs(real_poolpath)
+        self._files = {}
 
         c, t, u = self.readHK()
 
@@ -95,6 +95,26 @@ class LocalPool(ProductPool):
         self._tags.update(t)
         self._urns.update(u)
 
+    def readmmap(self, filename, close=False):
+        fp = op.abspath(filename)
+        try:
+            if 1:  # fp not in self._files or self._files[fp] is None:
+                file_obj = open(fp, mode="r+", encoding="utf-8")
+                # with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
+            else:
+                file_obj = self._files[fp]
+            # file_obj.seek(0)
+            js = file_obj.read()
+        except Exception as e:
+            msg = 'Error in HK reading ' + fp + str(e) + trbk(e)
+            logging.error(msg)
+            raise Exception(msg)
+        if 1:  # close:
+            file_obj.close()
+        else:
+            self._files[fp] = file_obj
+        return js
+
     def readHK(self, hktype=None, serialized=False):
         """
         loads and returns the housekeeping data
@@ -107,36 +127,62 @@ class LocalPool(ProductPool):
         else:
             hks = [hktype]
         fp0 = self.transformpath(self._poolname)
-        with filelock.FileLock(self.lockpath('w'), timeout=5):
-            # if 1:
-            hk = {}
-            for hkdata in hks:
-                fp = pathjoin(fp0, hkdata + '.jsn')
-                if op.exists(fp):
-                    try:
-                        with open(fp, 'r') as f:
-                            js = f.read()
-                    except Exception as e:
-                        msg = 'Error in HK reading ' + fp + str(e) + trbk(e)
-                        logging.error(msg)
-                        raise Exception(msg)
-                    r = js if serialized else deserialize(js)
-                else:
-                    r = '{}' if serialized else dict()
-                hk[hkdata] = r
+        # with filelock.FileLock(self.lockpath('w'), timeout=5):
+        # if 1:
+        hk = {}
+        for hkdata in hks:
+            fp = op.abspath(pathjoin(fp0, hkdata + '.jsn'))
+            if op.exists(fp):
+                js = self.readmmap(fp)
+                r = js if serialized else deserialize(js)
+            else:
+                r = '{}' if serialized else dict()
+            hk[hkdata] = r
+            assert r is not None
         logger.debug('HK read from ' + fp0)
         return (hk['classes'], hk['tags'], hk['urns']) if hktype is None else hk[hktype]
 
+    def writeJsonmmap(self, fp, data, close=False, **kwds):
+        """ write data in JSON from mmap file at fp.
+
+        register the file. Leave file open by default `close`.
+        data: to be serialized and saved.
+        """
+
+        # js = json.dumps(data, cls=ODEncoder)
+        #logger.debug('Writing %s stat %s' % (fp, str(os.path.exists(fp+'/..'))))
+        js = serialize(data, **kwds)
+        fp = op.abspath(fp)
+        if 1:  # fp not in self._files or self._files[fp] is None:
+            file_obj = open(fp, mode="w+", encoding="utf-8")
+            # with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_WRITE) as mmap_obj:
+        else:
+            file_obj = self._files[fp]
+        file_obj.seek(0)
+        # file_obj.resize(len(js))
+        file_obj.truncate(0)
+        file_obj.write(js)
+        # file_obj.flush()
+        close = 1
+        if close:
+            file_obj.close()
+        else:
+            self._files[fp] = file_obj
+        l = len(js)
+        logger.debug('JSON saved to: %s %d bytes' % (fp, l))
+        return l
+
     def writeHK(self, fp0):
         """
            save the housekeeping data to disk
         """
-
+        l = 0
         for hkdata in ['classes', 'tags', 'urns']:
             fp = pathjoin(fp0, hkdata + '.jsn')
-            writeJsonwithbackup(fp, self.__getattribute__('_' + hkdata))
+            l += self.writeJsonmmap(fp, self.__getattribute__('_' + hkdata))
+        return l
 
-    def schematicSave(self, resourcetype, index, data, tag=None):
+    def schematicSave(self, resourcetype, index, data, tag=None, **kwds):
         """
         does the media-specific saving.
 
@@ -145,38 +191,39 @@ class LocalPool(ProductPool):
         fp0 = self.transformpath(self._poolname)
         fp = pathjoin(fp0, quote(resourcetype) + '_' + str(index))
         try:
-            writeJsonwithbackup(fp, data)
-            self.writeHK(fp0)
+            #t0 = time.time()
+            l = self.writeJsonmmap(fp, data, close=True, **kwds)
+            l += self.writeHK(fp0)
+            #print('tl %.8f %9d' % (time.time()-t0, l))
             logger.debug('HK written')
         except IOError as e:
             logger.error('Save ' + fp + 'failed. ' + str(e) + trbk(e))
             raise e  # needed for undoing HK changes
+        return l
 
     def schematicLoadProduct(self, resourcetype, index, serialized=False):
         """
         does the scheme-specific part of loadProduct.
 
-        se
         """
+
         indexstr = str(index)
         pp = self.transformpath(self._poolname) + '/' + \
             resourcetype + '_' + indexstr
-        try:
-            with open(pp, 'r') as f:
-                js = f.read()
-        except Exception as e:
-            msg = 'Load' + pp + 'failed. ' + str(e) + trbk(e)
-            logger.error(msg)
-            raise e
+        js = self.readmmap(pp, close=True)
         return js if serialized else deserialize(js)
 
     def schematicRemove(self, resourcetype, index):
         """
-        does the scheme-specific part of removal.
+        does the scheme-specific part of removal of product from pool.
         """
         fp0 = self.transformpath(self._poolname)
-        fp = pathjoin(fp0,  quote(resourcetype) + '_' + str(index))
+        fp = op.abspath(pathjoin(fp0,  quote(resourcetype) + '_' + str(index)))
         try:
+            if fp in self._files and self._files[fp]:
+                self._files[fp].flush()
+                self._files[fp].close()
+                del self._files[fp]
             os.unlink(fp)
             self.writeHK(fp0)
         except IOError as e:
@@ -187,6 +234,10 @@ class LocalPool(ProductPool):
         """
         does the scheme-specific remove-all
         """
+        for n, f in self._files.items():
+            f.flush()
+            f.close()
+        self._files.clear()
         wipeLocal(self.transformpath(self._poolname))
 
     def getHead(self, ref):
